@@ -6,16 +6,17 @@ from datetime import datetime
 from collections import defaultdict
 
 from .base_rust_api import BaseRustSocket
+from .structures import RustTime, RustInfo, RustMap, RustMarker, RustChatMessage, RustTeamInfo, RustTeamMember, RustTeamNote, RustEntityInfo, RustContents, RustItem
+from .remote.rustplus_pb2 import *
+from .remote import HeartBeat
 from ..commands import CommandOptions
 from ..exceptions import *
 from ..utils import *
-from .structures import RustTime, RustInfo, RustMap, RustMarker, RustChatMessage, RustTeamInfo, RustTeamMember, RustTeamNote, RustEntityInfo, RustContents, RustItem
-from .remote.rustplus_pb2 import *
 
 class RustSocket(BaseRustSocket):
 
     def __init__(self, ip: str = None, port: str = None, steamid: int = None, playertoken: int = None, command_options : CommandOptions = None, raise_ratelimit_exception : bool = True, ratelimit_limit : int = 25, ratelimit_refill : int = 3) -> None:
-        super().__init__(ip=ip, port=port, steamid=steamid, playertoken=playertoken, command_options=command_options, raise_ratelimit_exception=raise_ratelimit_exception, ratelimit_limit=ratelimit_limit, ratelimit_refill=ratelimit_refill)
+        super().__init__(ip=ip, port=port, steamid=steamid, playertoken=playertoken, command_options=command_options, raise_ratelimit_exception=raise_ratelimit_exception, ratelimit_limit=ratelimit_limit, ratelimit_refill=ratelimit_refill, heartbeat=HeartBeat(self))
 
     async def get_time(self) -> RustTime:
         
@@ -24,9 +25,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getTime.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        response = await self.ws.get_response(app_request.seq)
+        response = await self.remote.sock().get_response(app_request.seq, app_request)
 
         return format_time(response)
 
@@ -40,9 +41,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.sendTeamMessage.CopyFrom(app_send_message)
 
-        self.ws.ignored_responses.append(app_request.seq)
+        self.remote.sock().ignored_responses.append(app_request.seq)
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
     async def get_info(self) -> RustInfo:
         
@@ -51,9 +52,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getInfo.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        response = await self.ws.get_response(app_request.seq)
+        response = await self.remote.sock().get_response(app_request.seq, app_request)
 
         return RustInfo(response.response.info)
 
@@ -64,9 +65,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getTeamChat.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        messages = (await self.ws.get_response(app_request.seq)).response.teamChat.messages
+        messages = (await self.remote.sock().get_response(app_request.seq, app_request)).response.teamChat.messages
 
         return [RustChatMessage(message) for message in messages]
 
@@ -77,9 +78,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getTeamInfo.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        app_message = await self.ws.get_response(app_request.seq)
+        app_message = await self.remote.sock().get_response(app_request.seq, app_request)
 
         return RustTeamInfo(app_message.response.teamInfo)
 
@@ -90,9 +91,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getMapMarkers.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        app_message = await self.ws.get_response(app_request.seq)
+        app_message = await self.remote.sock().get_response(app_request.seq, app_request)
 
         return [RustMarker(marker) for marker in app_message.response.mapMarkers.markers]
 
@@ -103,9 +104,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getMap.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        app_message = await self.ws.get_response(app_request.seq)
+        app_message = await self.remote.sock().get_response(app_request.seq, app_request)
 
         return RustMap(app_message.response.map)
 
@@ -118,9 +119,9 @@ class RustSocket(BaseRustSocket):
         app_request = self._generate_protobuf()
         app_request.getMap.CopyFrom(AppEmpty())
         
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        app_message = await self.ws.get_response(app_request.seq)
+        app_message = await self.remote.sock().get_response(app_request.seq, app_request)
 
         map = app_message.response.map
         monuments = list(map.monuments)
@@ -182,9 +183,9 @@ class RustSocket(BaseRustSocket):
         app_request.entityId = eid
         app_request.getEntityInfo.CopyFrom(AppEmpty())
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
-        app_message = await self.ws.get_response(app_request.seq)
+        app_message = await self.remote.sock().get_response(app_request.seq, app_request)
 
         return RustEntityInfo(app_message.response.entityInfo)
 
@@ -200,9 +201,9 @@ class RustSocket(BaseRustSocket):
         app_request.entityId = eid
         app_request.setEntityValue.CopyFrom(entityValue)
 
-        self.ws.ignored_responses.append(app_request.seq)
+        self.remote.sock().ignored_responses.append(app_request.seq)
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
     async def turn_on_smart_switch(self, eid: int = None) -> None:
         
@@ -223,15 +224,17 @@ class RustSocket(BaseRustSocket):
         if steamid is None:
             raise ValueError("SteamID cannot be None")
 
+        self._handle_ratelimit()
+
         leaderPacket = AppPromoteToLeader()
         leaderPacket.steamId = steamid
 
         app_request = self._generate_protobuf()
         app_request.promoteToLeader.CopyFrom(leaderPacket)
         
-        self.ws.ignored_responses.append(app_request.seq)
+        self.remote.sock().ignored_responses.append(app_request.seq)
 
-        await self.ws.send_message(app_request)
+        await self.remote.sock().send_message(app_request)
 
     async def get_current_events(self) -> List[RustMarker]:
         
