@@ -28,4 +28,8 @@ class ImageError(Error):
 
 class ClientNotConnectedError(Error):
     """Raised when you are not connected to the Rust server"""
+    pass
+
+class RequestError(Error):
+    """Raised when an error is recieved from the server"""
     pass
\ No newline at end of file
